<?php
/**
 * Created by JetBrains PhpStorm.
 * User: alanoliveira
 * Date: 28/03/14
 * Time: 15:13
 * To change this template use File | Settings | File Templates.
 */

namespace Financeiro\Logic\ContasPagar\Cnab\Remessa;

use Financeiro\Entities\Cnabpag;
use Financeiro\Util\Util;
use Fiscal\Entities\Boletpagar;
use Financeiro\Entities\Emprbanc;
use Geral\Entities\Tipobaixapag;
use Geral\Entities\Banco;



/**
 * Class GeraCnab
 * @package Financeiro\Logic\ContasPagar\Cnab\Remessa
 */
abstract class GeraCnab {

    CONST CODIGO_COMPENSACAO_SANTANDER = "033";
    CONST CODIGO_COMPENSACAO_BRADESCO  = "237";
    CONST CODIGO_COMPENSACAO_ITAU  = "341";
    CONST CODIGO_CONPENSACAO_CAIXA = "104";
    CONST CODIGO_COMPENSACAO_BRASIL = '001';

    CONST EXCLUSAO = 'E';

    protected $nrorg;
    /** @var $contcorr \Geral\Entities\Contcorr  */
    protected $contcorr;
    /** @var $filial \Geral\Entities\Filial */
    protected $filial;
    /** @var $empresa \Geral\Entities\Empresa */
    protected $empresa;
    /** @var $remessa \Financeiro\Entities\Cnabpag */
    protected $remessa;
    protected $inscricaojuridica;
    protected $nomeempresa;
    protected $tipoinscricao;


    /**
     *   Instancia a classe correspondente ao Banco da Remessa
     * @param Cnabpag $remessa
     * @return GeraCnabSantander
     * @throws \Exception
     */
    public static function factory(CnabPag $remessa)
    {
        try {

            $codigoCompensacao = $remessa->getReducont()->getAgencia()->getBanco()->getCdbanccnab();
            if ($codigoCompensacao === self::CODIGO_COMPENSACAO_SANTANDER) {
                return new GeraCnabSantander($remessa);
            }elseif($codigoCompensacao === self::CODIGO_COMPENSACAO_BRADESCO) {
                return new GeraCnabBradesco($remessa);
            }elseif($codigoCompensacao === self::CODIGO_COMPENSACAO_ITAU) {
                return new GeraCnabItau($remessa);
            }elseif($codigoCompensacao === self::CODIGO_CONPENSACAO_CAIXA)
                return new GeraCnabCaixa($remessa);
            elseif($codigoCompensacao === self::CODIGO_COMPENSACAO_BRASIL)
                return new GeraCnabBrasil($remessa);
            else throw new \Exception('O Códido de Compensação informado não pertece a nenhum banco reconhecido.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public abstract function geraHeaderArquivo();

    /**
     * @param $cdbanco
     * @internal param $formaLancamento
     * @return mixed
     */
    public abstract function geraHeaderLote($cdbanco, BoletPagar $boleto = null);

    public abstract function geraTraillerLote();

    public abstract function geraTrailerArquivo();

    /**
     * @param $caminhoArquivo
     * @param $boletos
     * @param bool $autorizaAgendamento - Parámetro utilizando quando o banco for Bradesco
     * @internal param $remessa
     * @return mixed
     */
    public abstract function geraArquivo($caminhoArquivo, $boletos, $autorizaAgendamento = false);




    /**
     * @param $string
     * @return mixed
     */
    protected function substitui_Sym_Chr(&$string){
        $sym = array(
            'À','Á','Ã','Â', 'à','á','ã','â',
            'Ê', 'É',
            'Í', 'í',
            'Ó','Õ','Ô', 'ó', 'õ', 'ô',
            'Ú','Ü',
            'Ç', 'ç',
            'é','ê',
            'ú','ü',
            'º', '&'
        );
        $chr = array(
            chr(192), chr(193), chr(195), chr(194), chr(224), chr(225), chr(227), chr(226),
            chr(202), chr(201),
            chr(205), chr(237),
            chr(211), chr(213), chr(212), chr(243), chr(245), chr(244),
            chr(218), chr(220),
            chr(199), chr(231),
            chr(233), chr(234),
            chr(250), chr(252),' ',' '
        );
        return str_replace($sym, $chr, $string);
    }

    /**
     * @return null
     */
    protected  function getIdentPag(){
        if($this->contcorr->getCdidpagcontcorr())
            return $this->contcorr->getCdidpagcontcorr();
        else{
            /** @var $emprbanc Emprbanc*/
            $emprbanc = Emprbanc::findOneByCdempresaAndCdbancoAndNrorg($this->empresa->getCdempresa(),
                                                                       $this->remessa->getCdbanco(),
                                                                       $this->nrorg
            );
            if($emprbanc and $emprbanc->getCdempbancpag())
                return $emprbanc->getCdempbancpag();
            else
                return null;
        }
    }

    /**
     * @param Boletpagar $boleto
     * @param null $idbaixautilp
     * @return string
     */
    protected function getBanco(Boletpagar $boleto, $idbaixautilp = null){
        if ($idbaixautilp == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO){
            return substr($boleto->getCdbarrapag(),0,3);
        }else{
            if($boleto->getCdbancbolp())
                return $boleto->getCdbancbolp();
            else
                return $boleto->getReducont()->getCdbanco();
        }
    }

    /**
     * @param $cdbanco
     * @param $nrorg
     * @return mixed
     */
    protected function getBancoForn($cdbanco, $nrorg){
        $banco = Banco::findOneByCdbanccnabAndNrorg($cdbanco, $nrorg);
        return $banco;
    }

    /** Retorna Conta Corrente sem a ultima posição (dígito)
     * @return string
     */
    protected function getContaCorrenteSemDigito(){
        $contacorrente = $this->contcorr->getCdcontcorr();
        return substr($contacorrente,0,-1);
    }

    /**
     * Retorna a ultima posição da Conta Corrente (Dígito)
     * @return int|string
     */
    protected function getDigitoContaCorrente(){
        $contacorrente = $this->contcorr->getCdcontcorr();
        return substr($contacorrente,-1);
    }






}